package java3.lang3;

public abstract class System {

	/*@ normal_behavior
	  @ requires src != null & dest != null;
	  @ requires length >= 0;
	  @ requires 0 <= srcPos && srcPos + length <= src.length;
	  @ requires 0 <= destPos && destPos + length <= dest.length;
	  @ assignable dest[*];
	  @ ensures (\forall int i; 0 <= i && i < destPos ==> dest[i] == \old(dest[i]));
	  @ ensures (\forall int i; 0 <= i && i < length ==> dest[destPos + i] == \old(src[srcPos + i]));
	  @ ensures (\forall int i; destPos + length <= i && i < dest.length; dest[i] == \old(dest[i]));
	  @ ensures src != dest ==> (\forall int i; 0 <= i && i < src.length ==> src[i] == \old(src[i]));
	  @
	  @ also	exceptional_behavior
	  @ requires src == null || dest == null;
	  @ signals (NullPointerException ex) src == null || dest == null;
	  @
	  @ also	exceptional_behavior
	  @ requires length < 0 || 
	  @          (srcPos < 0 || srcPos + length > src.length) || 
	  @          (destPos < 0 || destPos + length > dest.length);
	  @ signals (java3.lang3.IndexOutOfBoundsException ex)
	  @          length < 0 || 
	  @          (srcPos < 0 || srcPos + length > src.length) || 
	  @          (destPos < 0 || destPos + length > dest.length);
	  @*/
	public static void arraycopyInteger(java3.lang3.Integer[] src, int srcPos, java3.lang3.Integer[] dest,
			int destPos, int length) throws NullPointerException, java3.lang3.IndexOutOfBoundsException {

		if (src == null)
			throw new NullPointerException();

		if (dest == null)
			throw new NullPointerException();

		if (length < 0)
			throw new java3.lang3.IndexOutOfBoundsException();

		if (srcPos < 0 || srcPos + length > src.length)
			throw new java3.lang3.IndexOutOfBoundsException();

		if (destPos < 0 || destPos + length > dest.length)
			throw new java3.lang3.IndexOutOfBoundsException();

		if (src == dest) {
			// Use temporary array
			java3.lang3.Integer[] temp_array = new java3.lang3.Integer[length];
			//@ loop_invariant 0 <= i && i <= length;
			//@ loop_invariant temp_array != src;
			//@ loop_invariant (\forall int j; 0 <= j && j < i ==> temp_array[j] == src[srcPos + j]);
			//@ loop_invariant (\forall int j; 0 <= j && j < dest.length ==> dest[j] == \old(dest[j]));
			//@ loop_invariant (\forall int j; 0 <= j && j < src.length ==> src[j] == \old(src[j]));
			for (int i = 0; i < length; i++) {
				temp_array[i] = src[srcPos + i];
			}
			/*@ loop_invariant 0 <= i && i <= length;
			  @ loop_invariant (\forall int j; 0 <= j && j < i ==> dest[destPos + j] == temp_array[j]);
			  @ loop_invariant (\forall int j; 0 <= j && j < destPos ==> dest[j] == \old(dest[j]));
			  @ loop_invariant (\forall int j; 0 <= j && j < temp_array.length 
			  @                                             ==> temp_array[j] == \old(temp_array[j]));
			  @ loop_invariant (\forall int j; destPos + length <= j && j < dest.length
			  @                                             ==> dest[j] == \old(dest[j])); @*/
			for (int i = 0; i < length; i++) {
				dest[destPos + i] = temp_array[i];
			}
		} else {
			 /*@ loop_invariant 0 <= i && i <= length;
				@ loop_invariant (\forall int j; 0 <= j && j < src.length ==> src[j] == \old(src[j]));
				@ loop_invariant (\forall int j; 0 <= j && j < i ==> dest[destPos + j] == src[srcPos + j]);
				@ loop_invariant (\forall int j; 0 <= j && j < destPos ==> dest[j] == \old(dest[j]));
				@ loop_invariant (\forall int j; destPos + length <= j && j < dest.length 
				@                                            ==> dest[j] == \old(dest[j])); @*/
			for (int i = 0; i < length; i++) {
				dest[destPos + i] = src[srcPos + i];
			}
		}
	}

	/*@ normal_behavior
	  @ requires src != null & dest != null;
	  @ requires length >= 0;
	  @ requires srcPos >= 0 && srcPos + length <= src.length;
	  @ requires destPos >= 0 && destPos + length <= dest.length;
	  @ assignable src[*], dest[*];
	  @ ensures src != dest ==> (\forall int i; 0 <= i && i < src.length ==> src[i] == \old(src[i]));
	  @ ensures (\forall int i; 0 <= i && i < length ==> dest[destPos + i] == \old(src[srcPos + i]));
	  @ ensures (\forall int i; 0 <= i && i < destPos ==> dest[i] == \old(dest[i]));
	  @ ensures (\forall int i; destPos + length <= i && i < dest.length ==> dest[i] == \old(dest[i]));
	  @
	  @ also exceptional_behavior
	  @ requires src == null || dest == null;
	  @ signals (NullPointerException ex) src == null || dest == null;
	  @
	  @ also	exceptional_behavior
	  @ requires length < 0 || 
	  @          (srcPos < 0 || srcPos + length > src.length) || 
	  @          (destPos < 0 || destPos + length > dest.length);
	  @ signals (IndexOutOfBoundsException ex)
	  @			 length < 0 || 
	  @          (srcPos < 0 || srcPos + length > src.length) || 
	  @          (destPos < 0 || destPos + length > dest.length);
	  @*/
	public static void arraycopy(int[] src, int srcPos, int[] dest,
			int destPos, int length) throws NullPointerException, java3.lang3.IndexOutOfBoundsException {
		if (src == null)
			throw new NullPointerException();

		if (dest == null)
			throw new NullPointerException();

		if (length < 0)
			throw new java3.lang3.IndexOutOfBoundsException();

		if (srcPos < 0 || srcPos + length > src.length)
			throw new java3.lang3.IndexOutOfBoundsException();

		if (destPos < 0 || destPos + length > dest.length)
			throw new java3.lang3.IndexOutOfBoundsException();

		if (src == dest) {
			// Use temporary array
			int[] temp_array = new int[length];
			//@ loop_invariant 0 <= i && i <= length;
			//@ loop_invariant temp_array != src;
			//@ loop_invariant (\forall int j; 0 <= j && j < i ==> temp_array[j] == src[srcPos + j]);
			//@ loop_invariant (\forall int j; 0 <= j && j < dest.length ==> dest[j] == \old(dest[j]));
			//@ loop_invariant (\forall int j; 0 <= j && j < src.length ==> src[j] == \old(src[j]));
			for (int i = 0; i < length; i++) {
				temp_array[i] = src[srcPos + i];
			}
			/*@ loop_invariant 0 <= i && i <= length;
			  @ loop_invariant (\forall int j; 0 <= j && j < i ==> dest[destPos + j] == temp_array[j]);
			  @ loop_invariant (\forall int j; 0 <= j && j < destPos ==> dest[j] == \old(dest[j]));
			  @ loop_invariant (\forall int j; 0 <= j && j < temp_array.length 
			  @                                          ==> temp_array[j]==\old(temp_array[j]));
			  @ loop_invariant (\forall int j; destPos + length <= j && j < dest.length 
			  @                                          ==> dest[j] == \old(dest[j])); @*/
			for (int i = 0; i < length; i++) {
				dest[destPos + i] = temp_array[i];
			}
		} else {
			 /*@ loop_invariant 0 <= i && i <= length;
				@ loop_invariant (\forall int j; 0 <= j && j < src.length ==> src[j] == \old(src[j]));
				@ loop_invariant (\forall int j; 0 <= j && j < i ==> dest[destPos + j] == src[srcPos + j]);
				@ loop_invariant (\forall int j; 0 <= j && j < destPos ==> dest[j] == \old(dest[j]));
				@ loop_invariant (\forall int j; destPos + length <= j && j < dest.length 
				@                                         ==> dest[j] == \old(dest[j])); @*/
			for (int i = 0; i < length; i++) {
				dest[destPos + i] = src[srcPos + i];
			}
		}

	}

}
