package java3.theories;

public class TArrays {

	 // Is `a' empty?
	 /*@ 
		@ requires a != null;
		@
		@ ensures \result <==> (a.length == 0); 
		@*/
	 public static /*@ pure @*/ boolean empty(int[] a)
	 {
		  if (a == null)
				throw new RuntimeException();

		  return a.length == 0;
	 }

	 // Is x a valid position in `a'?
	 /*@ 
		@ requires a != null;
		@
		@ ensures \result <==> 0 <= x && x < a.length; 
		@*/
	 public static /*@ pure @*/ boolean in_bound(int[] a, int x)
	 {
		  if (a == null)
				throw new RuntimeException();

		  return (0 <= x && x < a.length);
	 }
	 
	 // Is `[fromIndex..toIndex)' a valid interval within `[0..a.length)'?
	 /*@ 
		@ requires a != null;
		@
		@ ensures \result <==> 0 <= fromIndex && fromIndex <= toIndex && toIndex <= a.length; 
		@*/
	 public static /*@ pure @*/ boolean within(int[] a, int fromIndex, int toIndex)
	 {
		  if (a == null)
				throw new RuntimeException();

		  return (0 <= fromIndex && fromIndex <= toIndex && toIndex <= a.length);
	 }

	 // Is `[fromIndex..toIndex)' a valid interval within `[0..a.length)'?
	 /*@ 
		@ requires a != null;
		@
		@ ensures \result <==> 0 <= fromIndex && fromIndex <= toIndex && toIndex <= a.length; 
		@*/
	 public static /*@ pure @*/ boolean within(java3.lang3.Integer[] a, int fromIndex, int toIndex)
	 {
		  if (a == null)
				throw new RuntimeException();

		  return (0 <= fromIndex && fromIndex <= toIndex && toIndex <= a.length);
	 }

	 // Is `a' sorted within `[fromIndex..toIndex)'?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i, j; fromIndex <= i && i < j && j < toIndex ==> a[i] <= a[j]); 
		@*/
	 public static /*@ pure @*/ boolean sorted(int[] a, int fromIndex, int toIndex)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (toIndex - fromIndex <= 1) {
				return true;
		  } else {
				int first = a[fromIndex], second = a[fromIndex + 1];
				return first <= second && sorted(a, fromIndex + 1, toIndex);
		  }
	 }

	 // Is `a' sorted within `[fromIndex..toIndex)'?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@ requires nonnull(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i, j; 
      @                 fromIndex <= i && i < j && j < toIndex ==> a[i].intValue() <= a[j].intValue()); 
		@*/
	 public static /*@ pure @*/ boolean sorted(java3.lang3.Integer[] a, int fromIndex, int toIndex)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (!nonnull(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (toIndex - fromIndex <= 1) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex], second = a[fromIndex + 1];
				return first.intValue() <= second.intValue() && sorted(a, fromIndex + 1, toIndex);
		  }
	 }

	 // Does `a' contain an element `key' within [fromIndex..toIndex)?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\exists int i; fromIndex <= i && i < toIndex && key == a[i]); 
		@*/
	 public static /*@ pure @*/ boolean in(int[] a, int fromIndex, int toIndex, int key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return false;
		  } else {
				int first = a[fromIndex];
				return (first == key) || in(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) < key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ 
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex ==> a[i] < key); 
		@*/
	 public static /*@ pure @*/ boolean less(int[] a, int fromIndex, int toIndex, int key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				int first = a[fromIndex];
				return (first < key) && less(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) <= key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex ==> a[i] <= key); 
		@*/
	 public static /*@ pure @*/ boolean lesseq(int[] a, int fromIndex, int toIndex, int key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				int first = a[fromIndex];
				return (first <= key) && lesseq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) <= key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@ requires nonnull(a, fromIndex, toIndex);
		@ requires key != null;
		@
		@ ensures \result <==> 
      @            (\forall int i; fromIndex <= i && i < toIndex ==> a[i].intValue() <= key.intValue());
		@*/
	 public static /*@ pure @*/ boolean lesseq(java3.lang3.Integer[] a, int fromIndex, int toIndex, 
															 java3.lang3.Integer key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (!nonnull(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (key == null)
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex];
				return (first.intValue() <= key.intValue()) && lesseq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) > key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex ==> a[i] > key); 
		@*/
	 public static /*@ pure @*/ boolean grt(int[] a, int fromIndex, int toIndex, int key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				int first = a[fromIndex];
				return (first > key) && grt(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) > key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@ requires nonnull(a, fromIndex, toIndex);
		@ requires key != null;
		@
		@ ensures \result <==> (\forall int i; 
		@                    fromIndex <= i && i < toIndex ==> a[i].intValue() > key.intValue()); 
		@*/
	 public static /*@ pure @*/ boolean grt(java3.lang3.Integer[] a, int fromIndex, int toIndex, 
														 java3.lang3.Integer key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (!nonnull(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (key == null)
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex];
				return (key.intValue() < first.intValue()) && grt(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) >= key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex ==> a[i] >= key); 
		@*/
	 public static  /*@ pure @*/ boolean grteq(int[] a, int fromIndex, int toIndex, int key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				int first = a[fromIndex];
				return (first >= key) && grteq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) >= key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@ requires nonnull(a, fromIndex, toIndex);
		@ requires key != null;
		@
		@ ensures \result <==> (\forall int i; 
      @                      fromIndex <= i && i < toIndex ==> a[i].intValue() >= key.intValue()); 
		@*/
	 public static  /*@ pure @*/ boolean grteq(java3.lang3.Integer[] a, int fromIndex, int toIndex, 
															 java3.lang3.Integer key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (!nonnull(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (key == null)
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex];
				return (first.intValue() >= key.intValue()) && grteq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[aFrom..aTo) == b[bFrom..bTo)?
	 /*@ 
		@ requires a != null && b != null;
		@ requires within(a, aFrom, aTo) && within(b, bFrom, bTo);
		@
		@ ensures \result <==> aTo - aFrom == bTo - bFrom && 
		@                      (\forall int i; aFrom <= i && i < aTo ==> a[i] == b[bFrom + (i - aFrom)]); 
		@*/
	 public static /*@ pure @*/ boolean eq(int[] a, int aFrom, int aTo, int[] b, int bFrom, int bTo)
	 {
		  if (a == null || b == null)
				throw new RuntimeException();
		  if (!within(a, aFrom, aTo) || !within(b, bFrom, bTo))
				throw new RuntimeException();

		  if (aTo - aFrom != bTo - bFrom) {
				return false;
		  } else { if (aFrom >= aTo) {
				return true;
				} else {
					 return (a[aFrom] == b[bFrom]) && eq(a, aFrom + 1, aTo, b, bFrom + 1, bTo);
				}
		  }
	 }

	 // Does a[fromIndex..toIndex) == key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex ==> a[i] == key); 
		@*/
	 public static /*@ pure @*/ boolean eq(int[] a, int fromIndex, int toIndex, int key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				int first = a[fromIndex];
				return (first == key) && eq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) == key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@ requires nonnull(a, fromIndex, toIndex);
		@ requires key != null;
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex 
		@                                 ==> a[i].intValue() == key.intValue()); 
		@*/
	 public static /*@ pure @*/ boolean eq(java3.lang3.Integer[] a, int fromIndex, int toIndex, 
														java3.lang3.Integer key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (!nonnull(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (key == null)
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex];
				return (first.intValue() == key.intValue()) && eq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) == null?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex 
		@                                 ==> a[i] == null); 
		@*/
	 public static /*@ pure @*/ boolean eqnull(java3.lang3.Integer[] a, int fromIndex, int toIndex)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex];
				return (first == null) && eqnull(a, fromIndex + 1, toIndex);
		  }
	 }

	 // Does a[fromIndex..toIndex) != key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex ==> a[i] != key); 
		@*/
	 public static /*@ pure @*/ boolean neq(int[] a, int fromIndex, int toIndex, int key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				int first = a[fromIndex];
				return (first != key) && neq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) != key?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@ requires nonnull(a, fromIndex, toIndex);
		@ requires key != null;
		@ 
		@ ensures \result <==> 
		@        (\forall int i; fromIndex <= i && i < toIndex ==> a[i].intValue() != key.intValue()); 
		@*/
	 public static /*@ pure @*/ boolean neq(java3.lang3.Integer[] a, int fromIndex, int toIndex, 
														 java3.lang3.Integer key)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (!nonnull(a, fromIndex, toIndex))
				throw new RuntimeException();
		  if (key == null)
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex];
				return (first.intValue() != key.intValue()) && neq(a, fromIndex + 1, toIndex, key);
		  }
	 }

	 // Does a[fromIndex..toIndex) != null?
	 /*@ 
		@ requires a != null;
		@ requires within(a, fromIndex, toIndex);
		@
		@ ensures \result <==> (\forall int i; fromIndex <= i && i < toIndex ==> a[i] != null); 
		@*/
	 public static /*@ pure @*/ boolean nonnull(java3.lang3.Integer[] a, int fromIndex, int toIndex)
	 {
		  if (a == null)
				throw new RuntimeException();
		  if (!within(a, fromIndex, toIndex))
				throw new RuntimeException();

		  if (fromIndex >= toIndex) {
				return true;
		  } else {
				java3.lang3.Integer first = a[fromIndex];
				return (first != null) && nonnull(a, fromIndex + 1, toIndex);
		  }
	 }

}