package java3.util3;

import java3.lang3.Integer;
import java3.theories.TLists;

/**
 * http://grepcode.com/file/repository.grepcode.com/java/root/jdk/openjdk/6-b14/java/util/ArrayDeque.java
 * @author galeotti
 */
public class ArrayDeque extends AbstractCollection {

	// CAF: Added to experiment with specification style, but then not really used
	/*@ 
	@ ensures \result.length == count();
	@ ensures head < tail ==> (\forall int i; 0 <= i && i < count() ==> \result[i] == elements[head + i]);
	@ ensures head > tail ==> 
	@     (\forall int i; 0 <= i && i < count() - tail ==> \result[i] == elements[tail + i]) &&
	@     (\forall int i; count() - tail <= i && i < count() 
	@                         ==> \result[i] == elements[i - count() + tail]);
	@ ensures head == tail ==> \result.length == 0;
	@*/
	public/*@ pure @*/java3.lang3.Integer[] active() {
		int ic = head, j = 0;
		java3.lang3.Integer el = elements[ic];
		int size = count();
		java3.lang3.Integer[] Result = new java3.lang3.Integer[size];

		/*@ loop_invariant el == elements[ic];
		 @
		 @ loop_invariant head <= tail ==> head <= ic && ic <= tail;
		 @ loop_invariant head <= tail ==> j == ic - head;
		 @ loop_invariant head <= tail ==> 
		 @              (\forall int k; 0 <= k && k < j ==> Result[k] == elements[head + k]);
		 @
		 @ loop_invariant head > tail ==> head <= ic && ic < elements.length || 0 <= ic && ic <= tail;
		 @ loop_invariant head > tail ==> ( head <= ic && ic < elements.length ==> j == ic - head );
		 @ loop_invariant head > tail ==> ( 0 <= ic && ic <= tail ==> j == ic + elements.length - head );
		 @ loop_invariant head > tail ==> ( head <= ic && ic < elements.length ==>
		 @                      (\forall int k; 0 <= k && k < j ==> Result[k] == elements[head + k]) );
		 @ loop_invariant head > tail ==> ( 0 <= ic && ic <= tail ==> 
		 @      (\forall int k; elements.length - head <= k && k < elements.length - head + j 
		 @                                    ==> Result[k] == elements[k - elements.length + head]) );
		 @
		 // Usual trick to avoid alleged violations of object invariant:
		 @ loop_invariant (\forall ArrayDeque ad; ad != null ==>
		 @   (\forall int k; 0 <= k && k < ad.elements.length ==> ad.elements[k] == \old(ad.elements[k])));
		 @*/
		while (ic != tail) {
			Result[j] = el;
			if (ic == elements.length) {
				ic = 0; // Wrap over
			} else {
				ic++;
			}
			el = elements[ic];
			j++;
		}
		return Result;
	}

	// CAF: Added for specification purposes
	/*@ 
	@ ensures 0 <= \result && \result <= elements.length;
	@ ensures head == tail ==> \result == 0;
	@ ensures head < tail ==> \result == tail - head;
	@ ensures head > tail ==> \result == (elements.length - head + tail); 
	@*/
	public/*@ pure @*/int count() {
		return ((head <= tail) ? tail - head : (elements.length - head + tail));
	}

	//@ invariant elements != null;
	//@ invariant 0 <= head && head < elements.length;
	//@ invariant 0 <= tail && tail < elements.length;
	//@ invariant elements[tail] == null;
	//@ invariant elements.owner == this;

	//@ invariant head < tail ==> TLists.neq(elements, head, tail, null);
	//@ invariant head < tail ==> TLists.eq(elements, 0, head, null);
	//@ invariant head < tail ==> TLists.eq(elements, tail, elements.length, null);

	//@ invariant tail < head ==> TLists.neq(elements, 0, tail, null);
	//@ invariant tail < head ==> TLists.neq(elements, head, elements.length, null);
	//@ invariant tail < head ==> TLists.neq(elements, tail, head, null);

	//@ invariant head == tail ==> TLists.eq(elements, 0, elements.length, null);

	/**
	 * The array in which the elements of the deque are stored.
	 * The capacity of the deque is the length of this array, which is
	 * always a power of two. The array is never allowed to become
	 * full, except transiently within an addX method where it is
	 * resized (see doubleCapacity) immediately upon becoming full,
	 * thus avoiding head and tail wrapping around to equal each
	 * other.  We also guarantee that all array cells not holding
	 * deque elements are always null.
	 */
	private java3.lang3.Integer[] elements;

	/**
	 * The index of the element at the head of the deque (which is the
	 * element that would be removed by remove() or pop()); or an
	 * arbitrary number equal to tail if the deque is empty.
	 */
	private int head;

	/**
	 * The index at which the next element would be added to the tail
	 * of the deque (via addLast(E), add(E), or push(E)).
	 */
	private int tail;

	/**
	 * The minimum capacity that we'll use for a newly created deque.
	 * Must be a power of 2.
	 */
	private static final int MIN_INITIAL_CAPACITY = 8;

	/**
	 * Removes the first occurrence of the specified element in this
	 * deque (when traversing the deque from head to tail).
	 * If the deque does not contain the element, it is unchanged.
	 * More formally, removes the first element <tt>e</tt> such that
	 * <tt>o.equals(e)</tt> (if such an element exists).
	 * Returns <tt>true</tt> if this deque contained the specified element
	 * (or equivalently, if this deque changed as a result of the call).
	 *
	 * @param value element to be removed from this deque, if present
	 * @return <tt>true</tt> if the deque contained the specified element
	 */
	// Focusing on the part of the specification interesting for the loop.
	/*@ 
	@ assignable head, tail, elements[*];
	@
	@ ensures !\result ==> (value == null || !TLists.deqHas(elements, head, tail, tail, value)) &&
	@               head == \old(head) && tail == \old(tail) && 
	@               (\forall int i; 0 <= i && i < elements.length ==> elements[i] == \old(elements[i]));
	@ ensures \result ==> value != null && \old(TLists.deqHas(elements, head, tail, tail, value));
		@*/
	public boolean removeFirstOccurrence(java3.lang3.Integer value)
			throws ConcurrentModificationException {
		int old_head = this.head;
		int old_tail = this.tail;
		java3.lang3.Integer[] old_elements = new java3.lang3.Integer[elements.length];
		for (int i = 0; i < elements.length; i++) {
			old_elements[i] = elements[i];
		}

		if (value == null)
			return false;

		int ic = head;
		java3.lang3.Integer x = elements[ic];

		int loop_counter_1 = 0;

		removeFirstOccurrence_loop_invariant_1(value, ic, elements, head, tail,
				x, old_head, old_tail, old_elements);
		while (x != null) {
			removeFirstOccurrence_loop_invariant_1(value, ic, elements, head,
					tail, x, old_head, old_tail, old_elements);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}

			if (value == x) {
				break;
			}
			if (ic == elements.length) {
				ic = 0;
			} else { // CAF: the following else branch was strangely missing but it's clearly needed...
				ic++;
			}
			x = elements[ic];
			removeFirstOccurrence_loop_invariant_1(value, ic, elements, head,
					tail, x, old_head, old_tail, old_elements);
		}
		removeFirstOccurrence_loop_invariant_1(value, ic, elements, head, tail,
				x, old_head, old_tail, old_elements);
		if (x != null) {
			delete(ic);
			return true;
		} else {
			return false;
		}
	}

	private static void removeFirstOccurrence_loop_invariant_1(
			java3.lang3.Integer value, int ic, java3.lang3.Integer[] elements,
			int head, int tail, java3.lang3.Integer x, int old_head,
			int old_tail, java3.lang3.Integer[] old_elements) {
		//$$stub:contains_loop_invariant_1$$
	}

	private/*@ pure @*/void checkInvariants() {
		boolean cond1 = elements[tail] == null;
		if (!cond1)
			throw new RuntimeException();

		boolean cond2 = (head == tail ? elements[head] == null
				: (elements[head] != null && elements[(tail - 1)
						& (elements.length - 1)] != null));
		if (!cond2)
			throw new RuntimeException();

		boolean cond3 = (elements[(head - 1) & (elements.length - 1)] == null);
		if (!cond3)
			throw new RuntimeException();

	}

	/**
	 * Removes the element at the specified position in the elements array,
	 * adjusting head and tail as necessary.  This can result in motion of
	 * elements backwards or forwards in the array.
	 *
	 * <p>This method is called delete rather than remove to emphasize
	 * that its semantics differ from those of {@link List#removeIndex(int)}.
	 *
	 * @return true if elements moved backwards
	 */
	/*@
	@ requires 0 <= index && index < elements.length;
	@ requires TLists.deqWithin(elements, head, tail, index);
	@
	@ assignable head, tail, elements[*];
	@ 
	@*/
	private boolean delete(int index) throws ConcurrentModificationException {
		checkInvariants();
		final java3.lang3.Integer[] elements = this.elements;
		final int mask = elements.length - 1;
		final int h = head;
		final int t = tail;
		final int front = (index - h) & mask;
		final int back = (t - index) & mask;

		// Invariant: head <= i < tail mod circularity
		if (front >= ((t - h) & mask))
			throw new ConcurrentModificationException();

		// Optimize for least element motion
		if (front < back) {
			if (h <= index) {
				java3.lang3.System.arraycopyInteger(elements, h, elements,
						h + 1, front);
			} else { // Wrap around
				java3.lang3.System.arraycopyInteger(elements, 0, elements, 1,
						index);
				elements[0] = elements[mask];
				java3.lang3.System.arraycopyInteger(elements, h, elements,
						h + 1, mask - h);
			}
			elements[h] = null;
			head = (h + 1) & mask;
			return false;
		} else {
			if (index < t) { // Copy the null tail as well
				java3.lang3.System.arraycopyInteger(elements, index + 1,
						elements, index, back);
				tail = t - 1;
			} else { // Wrap around
				java3.lang3.System.arraycopyInteger(elements, index + 1,
						elements, index, mask - index);
				elements[mask] = elements[0];
				java3.lang3.System
						.arraycopyInteger(elements, 1, elements, 0, t);
				tail = (t - 1) & mask;
			}
			return true;
		}
	}

	/*@ also 
	@ ensures !\result ==> (value == null || !TLists.deqHas(elements, head, tail, tail, value));
	@ ensures \result ==> value != null && TLists.deqHas(elements, head, tail, tail, value);
		@*/
	public/*@ pure @*/boolean contains(java3.lang3.Integer value) {
		if (value == null)
			return false;

		int ic = head;
		java3.lang3.Integer x = elements[ic];

		int loop_counter_1 = 0;

		contains_loop_invariant_1(value, ic, elements, x, head, tail);
		while (x != null) {
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			contains_loop_invariant_1(value, ic, elements, x, head, tail);
			if (value == x) {
				break;
			}
			if (ic == elements.length) {
				ic = 0;
			} else { // CAF: the following else branch was strangely missing but it's clearly needed...
				ic++;
			}
			x = elements[ic];
			contains_loop_invariant_1(value, ic, elements, x, head, tail);
		}
		contains_loop_invariant_1(value, ic, elements, x, head, tail);
		if (x != null) {
			return true;
		} else {
			return false;
		}
	}

	private static void contains_loop_invariant_1(java3.lang3.Integer value,
			int ic, java3.lang3.Integer[] elements, java3.lang3.Integer x,
			int head, int tail) {
		//$$stub:contains_loop_invariant_1$$
	}

	/**
	 * Removes all of the elements from this deque.
	 * The deque will be empty after this call returns.
	 */
	/*@ also
	  @ assignable head, tail, elements[*];
	  @
	  @ ensures TLists.eq(elements, 0, elements.length, null);
	  // Equivalent to the previous one, for uniformity of presentation
	  @ ensures \old(head) != \old(tail) ==> TLists.deqEq(elements, 0, elements.length, elements.length, null);
	  @ ensures head == tail;
	  @*/
	public void clear() {
		int old_head = this.head;
		int old_tail = this.tail;
		java3.lang3.Integer[] old_elements = new java3.lang3.Integer[elements.length];
		for (int i = 0; i < elements.length; i++) {
			old_elements[i] = elements[i];
		}

		int h = head;
		int t = tail;
		if (h != t) { // clear all cells
			head = tail = 0;
			int ic = h;

			int loop_counter_1 = 0;

			clear_loop_invariant_1(elements, head, tail, ic, h, t, old_head,
					old_tail, old_elements);
			while (ic != t) {
				if (loop_counter_1 < 10) {
					loop_counter_1++;
				}
				clear_loop_invariant_1(elements, head, tail, ic, h, t,
						old_head, old_tail, old_elements);
				elements[ic] = null;
				ic = (ic + 1);
				if (ic == elements.length)
					ic = 0;
				clear_loop_invariant_1(elements, head, tail, ic, h, t,
						old_head, old_tail, old_elements);
			}
			clear_loop_invariant_1(elements, head, tail, ic, h, t, old_head,
					old_tail, old_elements);
		}
	}

	private static void clear_loop_invariant_1(java3.lang3.Integer[] elements,
			int head, int tail, int ic, int h, int t, int old_head,
			int old_tail, java3.lang3.Integer[] old_elements) {
		//$$stub:clear_loop_invariant_1$$
	}

	/**
	 * Removes the last occurrence of the specified element in this
	 * deque (when traversing the deque from head to tail).
	 * If the deque does not contain the element, it is unchanged.
	 * More formally, removes the last element <tt>e</tt> such that
	 * <tt>o.equals(e)</tt> (if such an element exists).
	 * Returns <tt>true</tt> if this deque contained the specified element
	 * (or equivalently, if this deque changed as a result of the call).
	 *
	 * @param value element to be removed from this deque, if present
	 * @return <tt>true</tt> if the deque contained the specified element
	 * @throws ConcurrentModificationException 
	 */

	// Focusing on the part of the specification interesting for the loop.
	/*@ 
	@ assignable head, tail, elements[*];
	@
	@ ensures !\result ==> (value == null || !TLists.deqHas(elements, head, tail, tail, value)) &&
	@               head == \old(head) && tail == \old(tail) && 
	@               (\forall int i; 0 <= i && i < elements.length ==> elements[i] == \old(elements[i]));
	@ ensures \result ==> value != null && 
	@                     \old(TLists.deqHas(elements, head, tail, tail, value));
		@*/
	public boolean removeLastOccurrence(java3.lang3.Integer value)
			throws ConcurrentModificationException {
		int old_head = this.head;
		int old_tail = this.tail;
		java3.lang3.Integer[] old_elements = new java3.lang3.Integer[elements.length];
		for (int i = 0; i < elements.length; i++) {
			old_elements[i] = elements[i];
		}

		if (value == null)
			return false;

		int indexToRemove = -1;

		if (head < tail) {

			int ic = tail - 1;

			int loop_counter_1 = 0;

			removeLastOccurrence_loop_invariant_1(ic, tail, head, value,
					elements, old_head, old_tail, old_elements);
			for (; ic >= head;) {
				if (loop_counter_1 < 10) {
					loop_counter_1++;
				}

				removeLastOccurrence_loop_invariant_1(ic, tail, head, value,
						elements, old_head, old_tail, old_elements);
				if (value == elements[ic]) {
					break;
				}

				ic--;
				removeLastOccurrence_loop_invariant_1(ic, tail, head, value,
						elements, old_head, old_tail, old_elements);
			}
			removeLastOccurrence_loop_invariant_1(ic, tail, head, value,
					elements, old_head, old_tail, old_elements);
			if (ic >= head) {
				indexToRemove = ic;
			}

		} else {
			int ic = tail - 1;

			int loop_counter_2 = 0;

			removeLastOccurrence_loop_invariant_2(ic, tail, value, elements,
					old_head, old_tail, old_elements);

			for (; ic >= 0;) {
				if (loop_counter_2 < 10) {
					loop_counter_2++;
				}
				removeLastOccurrence_loop_invariant_2(ic, tail, value,
						elements, old_head, old_tail, old_elements);
				if (value == elements[ic])
					break;

				ic--;
				removeLastOccurrence_loop_invariant_2(ic, tail, value,
						elements, old_head, old_tail, old_elements);
			}
			removeLastOccurrence_loop_invariant_2(ic, tail, value, elements,
					old_head, old_tail, old_elements);

			if (ic >= 0) {
				indexToRemove = ic;
			} else {

				ic = elements.length - 1;

				int loop_counter_3 = 0;

				removeLastOccurrence_loop_invariant_3(ic, elements, head, tail,
						value, old_head, old_tail, old_elements);

				for (; ic >= head;) {
					if (loop_counter_3 < 10) {
						loop_counter_3++;
					}
					removeLastOccurrence_loop_invariant_3(ic, elements, head,
							tail, value, old_head, old_tail, old_elements);
					if (value == elements[ic])
						break;

					ic--;
					removeLastOccurrence_loop_invariant_3(ic, elements, head,
							tail, value, old_head, old_tail, old_elements);
				}
				removeLastOccurrence_loop_invariant_3(ic, elements, head, tail,
						value, old_head, old_tail, old_elements);
				if (ic >= head) {
					indexToRemove = ic;
				}

			}

		}

		if (indexToRemove != -1) {
			// A little nudge.
			//@ assert \old(TLists.deqHas(elements, head, tail, tail, value));
			delete(indexToRemove);
			return true;
		} else
			return false;
	}

	private static void removeLastOccurrence_loop_invariant_3(int ic,
			java3.lang3.Integer[] elements, int head, int tail,
			java3.lang3.Integer value, int old_head, int old_tail,
			java3.lang3.Integer[] old_elements) {
		//$$stub:removeLastOccurrence_loop_invariant_3$$
	}

	private static void removeLastOccurrence_loop_invariant_2(int ic, int tail,
			java3.lang3.Integer value, java3.lang3.Integer[] elements,
			int old_head, int old_tail, java3.lang3.Integer[] old_elements) {
		//$$stub:removeLastOccurrence_loop_invariant_2$$
	}

	private static void removeLastOccurrence_loop_invariant_1(int ic, int tail,
			int head, java3.lang3.Integer value,
			java3.lang3.Integer[] elements, int old_head, int old_tail,
			java3.lang3.Integer[] old_elements) {
		//$$stub:removeLastOccurrence_loop_invariant_1$$
	}

	/**
	 * Allocate empty array to hold the given number of elements.
	 *
	 * @param numElements  the number of elements to hold
	 */
	private void allocateElements(int numElements) {
		int initialCapacity = MIN_INITIAL_CAPACITY;
		// Find the best power of two to hold elements.
		// Tests "<=" because arrays aren't kept full.
		if (numElements >= initialCapacity) {
			initialCapacity = numElements;
			initialCapacity |= MathAux.unsigned_right_shift(initialCapacity, 1);
			initialCapacity |= MathAux.unsigned_right_shift(initialCapacity, 2);
			initialCapacity |= MathAux.unsigned_right_shift(initialCapacity, 4);
			initialCapacity |= MathAux.unsigned_right_shift(initialCapacity, 8);
			initialCapacity |= MathAux
					.unsigned_right_shift(initialCapacity, 16);
			initialCapacity++;

			if (initialCapacity < 0) // Too many elements, must back off
				initialCapacity = MathAux.unsigned_right_shift(initialCapacity,
						1);// Good luck allocating 2 ^ 30 elements
		}
		elements = new java3.lang3.Integer[initialCapacity];
	}

	/**
	 * Double the capacity of this deque.  Call only when full, i.e.,
	 * when head and tail have wrapped around to become equal.
	 */
	private void doubleCapacity() {
		int p = head;
		int n = elements.length;
		int r = n - p; // number of elements to the right of p
		int newCapacity = n << 1;
		if (newCapacity < 0)
			throw new IllegalStateException("Sorry, deque too big");
		java3.lang3.Integer[] a = new java3.lang3.Integer[newCapacity];
		java3.lang3.System.arraycopyInteger(elements, p, a, 0, r);
		java3.lang3.System.arraycopyInteger(elements, 0, a, r, p);
		elements = a;
		head = 0;
		tail = n;
	}

	/**
	 * Copies the elements from our element array into the specified array,
	 * in order (from first to last element in the deque).  It is assumed
	 * that the array is large enough to hold all elements in the deque.
	 *
	 * @return its argument
	 */
	private java3.lang3.Integer[] copyElements(java3.lang3.Integer[] a) {
		if (head < tail) {
			java3.lang3.System.arraycopyInteger(elements, head, a, 0, size());
		} else if (head > tail) {
			int headPortionLen = elements.length - head;
			java3.lang3.System.arraycopyInteger(elements, head, a, 0,
					headPortionLen);
			java3.lang3.System.arraycopyInteger(elements, 0, a, headPortionLen,
					tail);
		}
		return a;
	}

	/**
	 * Constructs an empty array deque with an initial capacity
	 * sufficient to hold 16 elements.
	 */
	public ArrayDeque() {
		elements = new java3.lang3.Integer[16];
	}

	/**
	 * Constructs an empty array deque with an initial capacity
	 * sufficient to hold the specified number of elements.
	 *
	 * @param numElements  lower bound on initial capacity of the deque
	 */
	public ArrayDeque(int numElements) {
		allocateElements(numElements);
	}

	/**
	 * Constructs a deque containing the elements of the specified
	 * collection, in the order they are returned by the collection's
	 * iterator.  (The first element returned by the collection's
	 * iterator becomes the first element, or <i>front</i> of the
	 * deque.)
	 *
	 * @param c the collection whose elements are to be placed into the deque
	 * @throws NullPointerException if the specified collection is null
	 */
	public ArrayDeque(Collection c) {
		allocateElements(c.size());
		addAll(c);
	}

	/**
	 * Inserts the specified element at the front of this deque.
	 *
	 * @param e the element to add
	 * @throws NullPointerException if the specified element is null
	 */
	public void addFirst(java3.lang3.Integer e) {
		if (e == null)
			throw new NullPointerException();
		elements[head = (head - 1) & (elements.length - 1)] = e;
		if (head == tail)
			doubleCapacity();
	}

	/**
	 * Inserts the specified element at the end of this deque.
	 *
	 * <p>This method is equivalent to {@link #add}.
	 *
	 * @param e the element to add
	 * @throws NullPointerException if the specified element is null
	 */
	public void addLast(java3.lang3.Integer e) {
		if (e == null)
			throw new NullPointerException();
		elements[tail] = e;
		if ((tail = (tail + 1) & (elements.length - 1)) == head)
			doubleCapacity();
	}

	/**
	 * Inserts the specified element at the front of this deque.
	 *
	 * @param e the element to add
	 * @return <tt>true</tt> (as specified by {@link Deque#offerFirst})
	 * @throws NullPointerException if the specified element is null
	 */
	public boolean offerFirst(java3.lang3.Integer e) {
		addFirst(e);
		return true;
	}

	/**
	 * Inserts the specified element at the end of this deque.
	 *
	 * @param e the element to add
	 * @return <tt>true</tt> (as specified by {@link Deque#offerLast})
	 * @throws NullPointerException if the specified element is null
	 */
	public boolean offerLast(java3.lang3.Integer e) {
		addLast(e);
		return true;
	}

	/**
	 * @throws NoSuchElementException {@inheritDoc}
	 */
	public java3.lang3.Integer removeFirst() {
		java3.lang3.Integer x = pollFirst();
		if (x == null)
			throw new NoSuchElementException();
		return x;
	}

	/**
	 * @throws NoSuchElementException {@inheritDoc}
	 */
	public java3.lang3.Integer removeLast() {
		java3.lang3.Integer x = pollLast();
		if (x == null)
			throw new NoSuchElementException();
		return x;
	}

	public java3.lang3.Integer pollFirst() {
		int h = head;
		java3.lang3.Integer result = elements[h]; // Element is null if deque empty
		if (result == null)
			return null;
		elements[h] = null; // Must null out slot
		head = (h + 1) & (elements.length - 1);
		return result;
	}

	public java3.lang3.Integer pollLast() {
		int t = (tail - 1) & (elements.length - 1);
		java3.lang3.Integer result = elements[t];
		if (result == null)
			return null;
		elements[t] = null;
		tail = t;
		return result;
	}

	/**
	 * @throws NoSuchElementException {@inheritDoc}
	 */
	public java3.lang3.Integer getFirst() {
		java3.lang3.Integer x = elements[head];
		if (x == null)
			throw new NoSuchElementException();
		return x;
	}

	/**
	 * @throws NoSuchElementException {@inheritDoc}
	 */
	public java3.lang3.Integer getLast() {
		java3.lang3.Integer x = elements[(tail - 1) & (elements.length - 1)];
		if (x == null)
			throw new NoSuchElementException();
		return x;
	}

	public java3.lang3.Integer peekFirst() {
		return elements[head]; // elements[head] is null if deque empty
	}

	public java3.lang3.Integer peekLast() {
		return elements[(tail - 1) & (elements.length - 1)];
	}

	/**
	 * Inserts the specified element at the end of this deque.
	 *
	 * <p>This method is equivalent to {@link #addLast}.
	 *
	 * @param e the element to add
	 * @return <tt>true</tt> (as specified by {@link Collection#add})
	 * @throws NullPointerException if the specified element is null
	 */
	public boolean add(java3.lang3.Integer e) {
		addLast(e);
		return true;
	}

	/**
	 * Inserts the specified element at the end of this deque.
	 *
	 * <p>This method is equivalent to {@link #offerLast}.
	 *
	 * @param e the element to add
	 * @return <tt>true</tt> (as specified by {@link Queue#offer})
	 * @throws NullPointerException if the specified element is null
	 */
	public boolean offer(java3.lang3.Integer e) {
		return offerLast(e);
	}

	/**
	 * Retrieves and removes the head of the queue represented by this deque.
	 *
	 * This method differs from {@link #poll poll} only in that it throws an
	 * exception if this deque is empty.
	 *
	 * <p>This method is equivalent to {@link #removeFirst}.
	 *
	 * @return the head of the queue represented by this deque
	 * @throws NoSuchElementException {@inheritDoc}
	 */
	public java3.lang3.Integer remove() {
		return removeFirst();
	}

	/**
	 * Retrieves and removes the head of the queue represented by this deque
	 * (in other words, the first element of this deque), or returns
	 * <tt>null</tt> if this deque is empty.
	 *
	 * <p>This method is equivalent to {@link #pollFirst}.
	 *
	 * @return the head of the queue represented by this deque, or
	 *         <tt>null</tt> if this deque is empty
	 */
	public java3.lang3.Integer poll() {
		return pollFirst();
	}

	/**
	 * Retrieves, but does not remove, the head of the queue represented by
	 * this deque.  This method differs from {@link #peek peek} only in
	 * that it throws an exception if this deque is empty.
	 *
	 * <p>This method is equivalent to {@link #getFirst}.
	 *
	 * @return the head of the queue represented by this deque
	 * @throws NoSuchElementException {@inheritDoc}
	 */
	public java3.lang3.Integer element() {
		return getFirst();
	}

	/**
	 * Retrieves, but does not remove, the head of the queue represented by
	 * this deque, or returns <tt>null</tt> if this deque is empty.
	 *
	 * <p>This method is equivalent to {@link #peekFirst}.
	 *
	 * @return the head of the queue represented by this deque, or
	 *         <tt>null</tt> if this deque is empty
	 */
	public java3.lang3.Integer peek() {
		return peekFirst();
	}

	/**
	 * Pushes an element onto the stack represented by this deque.  In other
	 * words, inserts the element at the front of this deque.
	 *
	 * <p>This method is equivalent to {@link #addFirst}.
	 *
	 * @param e the element to push
	 * @throws NullPointerException if the specified element is null
	 */
	public void push(java3.lang3.Integer e) {
		addFirst(e);
	}

	/**
	 * Pops an element from the stack represented by this deque.  In other
	 * words, removes and returns the first element of this deque.
	 *
	 * <p>This method is equivalent to {@link #removeFirst()}.
	 *
	 * @return the element at the front of this deque (which is the top
	 *         of the stack represented by this deque)
	 * @throws NoSuchElementException {@inheritDoc}
	 */
	public java3.lang3.Integer pop() {
		return removeFirst();
	}

	/**
	 * Returns the number of elements in this deque.
	 *
	 * @return the number of elements in this deque
	 */
	public int size() {
		return (tail - head) & (elements.length - 1);
	}

	/**
	 * Returns <tt>true</tt> if this deque contains no elements.
	 *
	 * @return <tt>true</tt> if this deque contains no elements
	 */
	public boolean isEmpty() {
		return head == tail;
	}

	/**
	 * Removes a single instance of the specified element from this deque.
	 * If the deque does not contain the element, it is unchanged.
	 * More formally, removes the first element <tt>e</tt> such that
	 * <tt>o.equals(e)</tt> (if such an element exists).
	 * Returns <tt>true</tt> if this deque contained the specified element
	 * (or equivalently, if this deque changed as a result of the call).
	 *
	 * <p>This method is equivalent to {@link #removeFirstOccurrence}.
	 *
	 * @param o element to be removed from this deque, if present
	 * @return <tt>true</tt> if this deque contained the specified element
	 */
	public boolean remove(java3.lang3.Integer o) {
		return removeFirstOccurrence(o);
	}

	/**
	 * Returns an array containing all of the elements in this deque
	 * in proper sequence (from first to last element).
	 *
	 * <p>The returned array will be "safe" in that no references to it are
	 * maintained by this deque.  (In other words, this method must allocate
	 * a new array).  The caller is thus free to modify the returned array.
	 *
	 * <p>This method acts as bridge between array-based and collection-based
	 * APIs.
	 *
	 * @return an array containing all of the elements in this deque
	 */
	public java3.lang3.Integer[] toArray() {
		return copyElements(new java3.lang3.Integer[size()]);
	}

	/**
	 * Returns a copy of this deque.
	 *
	 * @return a copy of this deque
	 */
	public ArrayDeque cloneMe() {
		try {
			ArrayDeque result = (ArrayDeque) super.clone();
			result.elements = Arrays.copyOf(elements, elements.length);
			return result;

		} catch (CloneNotSupportedException e) {
			throw new AssertionError();
		}
	}

	/**
	 * Serialize this deque.
	 *
	 * @serialData The current size (<tt>int</tt>) of the deque,
	 * followed by all of its elements (each an object reference) in
	 * first-to-last order.
	 *
	 */
	/*
	private void writeObject(ObjectOutputStream s) throws IOException {
		s.defaultWriteObject();

		// Write out size
		s.writeInt(size());

		// Write out elements in order.
		int mask = elements.length - 1;
		for (int i = head; i != tail; i = (i + 1) & mask)
			s.writeObject(elements[i]);
	}
	*/

	/**
	 * Deserialize this deque.
	 */
	/*
	private void readObject(ObjectInputStream s) throws IOException,
			ClassNotFoundException {
		s.defaultReadObject();

		// Read in size and allocate array
		int size = s.readInt();
		allocateElements(size);
		head = 0;
		tail = size;

		// Read in all elements in the proper order.
		for (int i = 0; i < size; i++)
			elements[i] = (java3.lang3.Integer) s.readObject();
	}
	*/

	/**
	 * Returns an iterator over the elements in this deque.  The elements
	 * will be ordered from first (head) to last (tail).  This is the same
	 * order that elements would be dequeued (via successive calls to
	 * {@link #remove} or popped (via successive calls to {@link #pop}).
	 *
	 * @return an iterator over the elements in this deque
	 */
	public Iterator iterator() {
		return new DeqIterator();
	}

	public Iterator descendingIterator() {
		return new DescendingIterator();
	}

	/**
	 * The only purpose of this method is allowing 
	 * Chicory to access the private fields of this class.
	 * 
	 */
	private void unreachable_method() {
	}

	private class DeqIterator implements Iterator {
		/**
		 * Index of element to be returned by subsequent call to next.
		 */
		private int cursor = head;

		/**
		 * Tail recorded at construction (also in remove), to stop
		 * iterator and also to check for comodification.
		 */
		private int fence = tail;

		/**
		 * Index of element returned by most recent call to next.
		 * Reset to -1 if element is deleted by a call to remove.
		 */
		private int lastRet = -1;

		public boolean hasNext() {
			return cursor != fence;
		}

		public java3.lang3.Integer next() {
			if (cursor == fence)
				throw new NoSuchElementException();
			java3.lang3.Integer result = elements[cursor];
			// This check doesn't catch all possible comodifications,
			// but does catch the ones that corrupt traversal
			if (tail != fence || result == null)
				throw new ConcurrentModificationException();
			lastRet = cursor;
			cursor = (cursor + 1) & (elements.length - 1);
			return result;
		}

		public void remove() {
			if (lastRet < 0)
				throw new IllegalStateException();
			if (delete(lastRet)) { // if left-shifted, undo increment in next()
				cursor = (cursor - 1) & (elements.length - 1);
				fence = tail;
			}
			lastRet = -1;
		}
	}

	private class DescendingIterator implements Iterator {
		/*
		 * This class is nearly a mirror-image of DeqIterator, using
		 * tail instead of head for initial cursor, and head instead of
		 * tail for fence.
		 */
		private int cursor = tail;
		private int fence = head;
		private int lastRet = -1;

		public boolean hasNext() {
			return cursor != fence;
		}

		public java3.lang3.Integer next() {
			if (cursor == fence)
				throw new NoSuchElementException();
			cursor = (cursor - 1) & (elements.length - 1);
			java3.lang3.Integer result = elements[cursor];
			if (head != fence || result == null)
				throw new ConcurrentModificationException();
			lastRet = cursor;
			return result;
		}

		public void remove() {
			if (lastRet < 0)
				throw new IllegalStateException();
			if (!delete(lastRet)) {
				cursor = (cursor + 1) & (elements.length - 1);
				fence = head;
			}
			lastRet = -1;
		}
	}

}
